//+------------------------------------------------------------------+
//|                                                      DA_Demo.mq5 |
//|                        Copyright 2011, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Artem Galeev"
#property link      "http://www.mql5.com"
#property version   "1.00"
//--- indicator input parameters
input int                BearsPower_PeriodBears=13;          // Bears Power period
input int                BullsPower_PeriodBulls=13;          // Bulls Power period 
input int                CCI_PeriodCCI         =8;           // CCI period
input ENUM_APPLIED_PRICE CCI_Applied           =PRICE_CLOSE; // CCI applied price
input int                DeM_PeriodDeM         =8;           // DeMarker period
input int                FraMA_PeriodMA        =12;          // FraMA period
input int                FraMA_Shift           =0;           // FraMA shift
input ENUM_APPLIED_PRICE FraMA_Applied         =PRICE_CLOSE; // FraMA applied price
input int                MACD_PeriodFast       =12;          // MACD fast MA period
input int                MACD_PeriodSlow       =24;          // MACD slow MA period
input int                MACD_PeriodSignal     =9;           // MACD signal line period
input ENUM_APPLIED_PRICE MACD_Applied          =PRICE_CLOSE; // MACD applied price
input int                RSI_PeriodRSI         =8;           // RSI period
input ENUM_APPLIED_PRICE RSI_Applied           =PRICE_CLOSE; // RSI applied price
input int                RVI_PeriodRVI         =10;          // RVI period
input int                Stoch_PeriodK         =8;           // Stochastic K-period
input int                Stoch_PeriodD         =3;           // Stochastic D-period
input int                Stoch_PeriodSlow      =3;           // Stochastic Slowing period
input ENUM_STO_PRICE     Stoch_Applied         =STO_LOWHIGH; // Stochastic applied price
input int                WPR_PeriodWPR         =8;           // WPR period
//--- indicator handles
int h_AC,h_BearsPower,h_BullsPower,h_AO,h_CCI,h_DeMarker,h_FrAMA;
int h_MACD,h_RSI,h_RVI,h_Stoch,h_WPR;
//--- arrays
double buf_AC[],buf_BearsPower[],buf_BullsPower[],buf_AO[],buf_CCI[];
double buf_DeMarker[],buf_FrAMA[],buf_MACD_m[],buf_MACD_s[],buf_RSI[];
double buf_RVI_m[],buf_RVI_s[],buf_Stoch_m[],buf_Stoch_s[],buf_WPR[];

//--- file handle
int FileHandle=-1;
//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- initialization of indicators
   h_AC=iAC(Symbol(),Period());
   h_BearsPower=iBearsPower(Symbol(),Period(),BearsPower_PeriodBears);
   h_BullsPower=iBullsPower(Symbol(),Period(),BullsPower_PeriodBulls);
   h_AO=iAO(Symbol(),Period());
   h_CCI=iCCI(Symbol(),Period(),CCI_PeriodCCI,CCI_Applied);
   h_DeMarker=iDeMarker(Symbol(),Period(),DeM_PeriodDeM);
   h_FrAMA=iFrAMA(Symbol(),Period(),FraMA_PeriodMA,FraMA_Shift,FraMA_Applied);
   h_MACD=iMACD(Symbol(),Period(),MACD_PeriodFast,MACD_PeriodSlow,MACD_PeriodSignal,MACD_Applied);
   h_RSI=iRSI(Symbol(),Period(),RSI_PeriodRSI,RSI_Applied);
   h_RVI=iRVI(Symbol(),Period(),RVI_PeriodRVI);
   h_Stoch=iStochastic(Symbol(),Period(),Stoch_PeriodK,Stoch_PeriodD,Stoch_PeriodSlow,MODE_SMA,Stoch_Applied);
   h_WPR=iWPR(Symbol(),Period(),WPR_PeriodWPR);

   if(h_AC==INVALID_HANDLE || h_BearsPower==INVALID_HANDLE || 
      h_BullsPower==INVALID_HANDLE || h_AO==INVALID_HANDLE || 
      h_CCI==INVALID_HANDLE || h_DeMarker==INVALID_HANDLE || 
      h_FrAMA==INVALID_HANDLE || h_MACD==INVALID_HANDLE || 
      h_RSI==INVALID_HANDLE || h_RVI==INVALID_HANDLE || 
      h_Stoch==INVALID_HANDLE || h_WPR==INVALID_HANDLE)
     {
      Print("Error creating indicators");
      return(1);
     }

   ArraySetAsSeries(buf_AC,true);
   ArraySetAsSeries(buf_BearsPower,true);
   ArraySetAsSeries(buf_BullsPower,true);
   ArraySetAsSeries(buf_AO,true);
   ArraySetAsSeries(buf_CCI,true);
   ArraySetAsSeries(buf_DeMarker,true);
   ArraySetAsSeries(buf_FrAMA,true);
   ArraySetAsSeries(buf_MACD_m,true);
   ArraySetAsSeries(buf_MACD_s,true);
   ArraySetAsSeries(buf_RSI,true);
   ArraySetAsSeries(buf_RVI_m,true);
   ArraySetAsSeries(buf_RVI_s,true);
   ArraySetAsSeries(buf_Stoch_m,true);
   ArraySetAsSeries(buf_Stoch_s,true);
   ArraySetAsSeries(buf_WPR,true);


   FileHandle=FileOpen("MasterData2.csv",FILE_ANSI|FILE_WRITE|FILE_CSV|FILE_SHARE_READ,';');
   if(FileHandle!=INVALID_HANDLE)
     {
      Print("FileOpen OK");
      //--- saving names of the variables in the first line of the file for convenience of working with it
      FileWrite(FileHandle,"Time","Hour","Price","AC","dAC","Bears","dBears","Bulls","dBulls",
                "AO","dAO","CCI","dCCI","DeMarker","dDeMarker","FrAMA","dFrAMA","MACDm","dMACDm",
                "MACDs","dMACDs","MACDms","dMACDms","RSI","dRSI","RVIm","dRVIm","RVIs","dRVIs",
                "RVIms","dRVIms","Stoch_m","dStoch_m","Stoch_s","dStoch_s","Stoch_ms","dStoch_ms",
                "WPR","dWPR");
     }
   else
     {
      Print("FileOpen action failed. Error ",GetLastError());
      ExpertRemove();
     }
//---
   return(0);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
   if(FileHandle!=INVALID_HANDLE) FileClose(FileHandle);
//---
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//| Monitoring the market situation and saving values                |
//| of the indicators into the file at the beginning of every new bar                  |
//+------------------------------------------------------------------+
void OnTick()
  {
//--- declaring a static variable of datetime type
   static datetime Prev_time;

//--- it will be used to store prices, volumes and spread of each bar
   MqlRates mrate[];
   MqlTick tickdata;

   ArraySetAsSeries(mrate,true);    
   
//--- obtaining the recent quotes
   if(!SymbolInfoTick(_Symbol,tickdata))
     {
      Alert("Quote update error - error: ",GetLastError(),"!!");
      return;
     }
///--- copying data of the last 4 bars
   if(CopyRates(_Symbol,_Period,0,4,mrate)<0)
     {
      Alert("Historical quote copy error - error: ",GetLastError(),"!!");
      return;
     }
//--- if both time values are equal, there is no new bar
   if(Prev_time==mrate[0].time) return;
//--- saving the time in the static variable 
   Prev_time=mrate[0].time;
 
//--- filling the arrays with values of the indicators
   bool copy_result=true;
   copy_result=copy_result && FillArrayFromBuffer1(buf_AC,h_AC,4);
   copy_result=copy_result && FillArrayFromBuffer1(buf_BearsPower,h_BearsPower,4);
   copy_result=copy_result && FillArrayFromBuffer1(buf_BullsPower,h_BullsPower,4);
   copy_result=copy_result && FillArrayFromBuffer1(buf_AO,h_AO,4);
   copy_result=copy_result && FillArrayFromBuffer1(buf_CCI,h_CCI,4);
   copy_result=copy_result && FillArrayFromBuffer1(buf_DeMarker,h_DeMarker,4);
   copy_result=copy_result && FillArrayFromBuffer1(buf_FrAMA,h_FrAMA,4);
   copy_result=copy_result && FillArraysFromBuffers2(buf_MACD_m,buf_MACD_s,h_MACD,4);
   copy_result=copy_result && FillArrayFromBuffer1(buf_RSI,h_RSI,4);
   copy_result=copy_result && FillArraysFromBuffers2(buf_RVI_m,buf_RVI_s,h_RVI,4);
   copy_result=copy_result && FillArraysFromBuffers2(buf_Stoch_m,buf_Stoch_s,h_Stoch,4);
   copy_result=copy_result && FillArrayFromBuffer1(buf_WPR,h_WPR,4);

//--- checking the accuracy of copying the data
   if(!copy_result==true)
     {
      Print("Data copy error");
      return;
     }

//--- saving to the file the price movement within the last two bars 
//--- and the preceding values of the indicators 
   if(FileHandle!=INVALID_HANDLE)
     {
      MqlDateTime tm;
      TimeCurrent(tm);
      uint Result=0;
      Result=FileWrite(FileHandle,TimeToString(TimeCurrent()),tm.hour,// time of the bar
                       (mrate[1].close-mrate[2].close)/_Point,// difference between the closing prices of the last two bars 
                       buf_AC[2],buf_AC[2]-buf_AC[3],// value of the indicator on the preceding bar and its dynamics
                       buf_BearsPower[2],buf_BearsPower[2]-buf_BearsPower[3],
                       buf_BullsPower[2],buf_BullsPower[2]-buf_BullsPower[3],
                       buf_AO[2],buf_AO[2]-buf_AO[3],
                       buf_CCI[2],buf_CCI[2]-buf_CCI[3],
                       buf_DeMarker[2],buf_DeMarker[2]-buf_DeMarker[3],
                       buf_FrAMA[2],buf_FrAMA[2]-buf_FrAMA[3],
                       buf_MACD_m[2],buf_MACD_m[2]-buf_MACD_m[3],
                       buf_MACD_s[2],buf_MACD_s[2]-buf_MACD_s[3],
                       buf_MACD_m[2]-buf_MACD_s[2],buf_MACD_m[2]-buf_MACD_s[2]-buf_MACD_m[3]+buf_MACD_s[3],
                       buf_RSI[2],buf_RSI[2]-buf_RSI[3],
                       buf_RVI_m[2],buf_RVI_m[2]-buf_RVI_m[3],
                       buf_RVI_s[2],buf_RVI_s[2]-buf_RVI_s[3],
                       buf_RVI_m[2]-buf_RVI_s[2],buf_RVI_m[2]-buf_RVI_s[2]-buf_RVI_m[3]+buf_RVI_s[3],
                       buf_Stoch_m[2],buf_Stoch_m[2]-buf_Stoch_m[3],
                       buf_Stoch_s[2],buf_Stoch_s[2]-buf_Stoch_s[3],
                       buf_Stoch_m[2]-buf_Stoch_s[2],buf_Stoch_m[2]-buf_Stoch_s[2]-buf_Stoch_m[3]+buf_Stoch_s[3],
                       buf_WPR[2],buf_WPR[2]-buf_WPR[3]);

      if(Result==0)
        {
         Print("FileWrite action error ",GetLastError());
         ExpertRemove();
        }
     }

  }
//+------------------------------------------------------------------+
//| Filling the indicator buffer from the single buffer indicator       |
//+------------------------------------------------------------------+
bool FillArrayFromBuffer1(double &buffer[],  // indicator buffer of values 
                          int ind_handle,    // indicator handle 
                          int amount)        // number of copied values
  {
//--- resetting error code
   ResetLastError();
//--- filling a part of the buffer array with the values from the indicator buffer with index 0
   if(CopyBuffer(ind_handle,0,0,amount,buffer)<0)
     {
      //--- if the copying fails, report the error code
      PrintFormat("Failed to copy data from the indicator, error code %d",GetLastError());
      //--- quitting with zero result - it means that the indicator will be considered as not calculated
      return(false);
     }
//--- success
   return(true);
  }
//+------------------------------------------------------------------+
//| Filling the indicator buffers from the double buffer indicator     |
//+------------------------------------------------------------------+
bool FillArraysFromBuffers2(double &buffer1[], // indicator buffer 1
                            double &buffer2[], // indicator buffer 2 
                            int ind_handle,    // indicator handle
                            int amount)        // number of copied values
  {
//--- resetting error code
   ResetLastError();
//--- filling a part of the buffer1 array with the values from the indicator buffer with index 0
   if(CopyBuffer(ind_handle,0,0,amount,buffer1)<0)
     {
      //--- if the copying fails, report the error code
      PrintFormat("Failed to copy data from the indicator buffer 1, error code %d",GetLastError());
      //--- quitting with zero result - it means that the indicator will be considered as not calculated
      return(false);
     }

//--- filling a part of the buffer2 array with the values from the indicator buffer with index 1
   if(CopyBuffer(ind_handle,1,0,amount,buffer2)<0)
     {
      //--- if the copying fails, report the error code
      PrintFormat("Failed to copy data from the indicator buffer 2, error code %d",GetLastError());
      //--- quitting with zero result - it means that the indicator will be considered as not calculated
      return(false);
     }
//--- success
   return(true);
  }
//+------------------------------------------------------------------+